@@ -2,7 +2,7 @@
   Licensed under the GNU Public License.
   Copyright (C) 1998-99 by Thomas M. Vier, Jr. All Rights Reserved.
 
-  wipe v0.58.4
+  wipe v0.58.6
   by Tom Vier <thomassr@erols.com> for Tortured Souls Software
 
   http://www.erols.com/thomassr/zero/tortured_souls.html
@@ -14,7 +14,7 @@
   freely modify and redistribute as per license.
 */
 
-#define VERSION "0.58.4"
+#define VERSION "0.58.6"
 #define COPYRIGHT "Copyright (C) 1998-99 Thomas M. Vier, Jr. All Rights Reserved."
 
 #define EMAIL "thomassr@erols.com"
@@ -82,7 +82,7 @@ int random_source = -1;      /* /dev/urandom */
 
 unsigned char custom_byte;   /* custom overwrite byte */
 
-char *hashin = NULL;         /* hash input buffer */
+char *hashin = NULL;         /* hash input buffer -- must be chars for hashin_update()*/
 size_t hashin_size = HASHIN_SIZE;
 
 char *buffer = NULL;         /* file buffer */
@@ -267,6 +267,7 @@ char gettrn_char(void)
 int init_hashin(void)
 {
   static _pword hash_seed; /* secret xor seed */
+  int saved;
 
   if (hashin == NULL)
     {
@@ -288,21 +289,18 @@ int init_hashin(void)
 void hashin_update(void)
 {
   static _pword hash_seed; /* secret xor seed */
-  off_t hashin_off = 0;
   _pword *buf; /* pointer to hashin */
+  off_t hashin_off = 0, end;
 
   if (options.seclevel)
     read(random_source, &hash_seed, sizeof(hash_seed)); /* get a new seed */
 
   buf = (_pword *) hashin;
+  end = (hashin_size * sizeof(char)) / sizeof(_pword); /* scale it */
 
   /* xor a (processor) word at a time */
-
-  while (hashin_off < hashin_size)
-    { 
-      buf[hashin_off] ^= hash_seed;  /* xor unknown data with an unknown seed */
-      hashin_off += sizeof(hash_seed);
-    }
+  while (hashin_off < end)
+    buf[hashin_off++] ^= hash_seed;  /* xor unknown data with an unknown seed */
 }
 
 /*
@@ -314,47 +312,21 @@ void hashin_update(void)
 
 void hashout_update(void)
 {
-  char *buf;
-  off_t offset = 0;
-  const int incr = sizeof(__u64) * 3; /* buf and offset increment value */
+  __u64 *buf, *bufend;
+
+  buf = (__u64 *) buffer;
+  bufend = buf; bufend += (buffer_size * sizeof(char)) / sizeof(__u64);
 
   /*
     keep getting three 64-bit values from tiger
     until the buffer is full
   */
 
-  buf = buffer;
-
-  /*
-    i'm using two variables to keep track of were in *buffer we are
-    cuz i figured two const increments are faster than one minus,
-    in the while loop.
-
-    it may be slower on register starved processors, but it should
-    be especially fast on alpha's since they can scale increments.
-    as long as egcs/gcc supports that.
-
-    it's not a huge improvement, but it seems to shave off a couple
-    seconds or so on a 73728 byte file on my 80MHz powerpc 601
-
-    please tell me if this is slower on x86 and other systems.
-  */
-
-  while (offset < buffer_size)
+  while (buf < bufend)
     {
-      /*
-	ok, what i did here is skip the middle man and just write
-	directly to the buffer
-      */
       hashin_update(); /* secret xor junx */
-
-      /* hash into the file buffer */
-      /* tiger returns a */
-      tiger((__u8 *) hashin, sizeof(hashin), (__u64 *) buf);
-
-      /* increment */
-      buf += incr;
-      offset += incr;
+      tiger((__u8 *) hashin, sizeof(hashin), buf); /* zero copy */
+      buf += 3;
     }
 }
 
@@ -745,6 +717,8 @@ int wipe(void)
 
   if ((buffer = realloc(buffer, buffer_size)) == NULL)
     {
+      fprintf(stderr, "\rwipe(): malloc returned NULL -- %ld\n", buffer_size);
+      abort();
       fprintf(stderr, "\r%s: %s: cannot allocate %ld bytes for file buffer\n", 
 	      program_name, strerror(errno), filestat.st_size, pathname);
       exit_code = errno; return(errno);
@@ -837,6 +811,7 @@ int wipe(void)
 
 /*
   lock -- create as good a lock as possible
+          what a mess
 */
 
 #ifdef LOCKABLE
@@ -946,7 +921,7 @@ int getfiledata(char *file)
      instead of what they point to */
   if (lstat(file, &filestat) != 0)
     {
-      fprintf(stderr, "\r%s: %s: cannot read inode: %s\n", 
+      fprintf(stderr, "\r%s: %s: %s\n", 
 	      program_name, strerror(errno), file);
       exit_code = errno; return(errno);
     }
@@ -1080,59 +1055,6 @@ int drilldown(const char str[])
   return(0);
 }
 
-/*
-  direntcheck -- see if file exists
-
-  dive (EFnet #reet) suggested scanning the dirents
-  instead of using access() to see if a file exists
-*/
-int direntcheck(const char str[])
-{
-  DIR *dir;
-  struct dirent *entry;
-  char *base, file[NAME_MAX+1];
-  int len;
-
-  /* dirents never end with slashes -- strip 'em */
-  strncpy(file, str, sizeof(file));
-
-  len = strlen(file); --len;
-  base = (char *) ((file + len)); /* find the last char */
-
-  /* loop it, in case of multiple slashes */
-  while (strncmp((char *) base, "/", 1) == 0)
-    {
-      file[len] = (char) 0;
-
-      len = strlen(file); --len;
-      base = (char *) ((file + len));
-    }
-
-  if ((dir = opendir(".")) == NULL)
-    {
-      fprintf(stderr, "\r%s: %s: cannot open directory: .\n", 
-	      program_name, strerror(errno));
-      exit_code = errno; return(errno);
-    }
-
-  len = sizeof(entry->d_name);
-
-  while ((entry = readdir(dir)) != NULL)
-    {
-      if (strncmp(entry->d_name, ".", len) && strncmp(entry->d_name, "..", len) != 0)
-	if (strncmp(entry->d_name, file, strlen(file)) == 0)
-	  {
-#ifdef DIRENT_TEST
-	    printf("found match: %s == %s\n", file, entry->d_name);
-#endif
-	    closedir(dir); return(0); /* found file in the directory */
-	  }
-    }
-
-  closedir(dir);
-  return(1); /* file not found - does not exist */
-}
-
 /*
   checkfile -- check access and destroy(), if ok
 */
@@ -1141,15 +1063,6 @@ int checkfile(char file[])
 {
   int code;
 
-  /* see if file exists */
-  if (direntcheck(file))
-    {
-      if (options.force) return(0);
-      fprintf(stderr, "\r%s: %s\n", 
-	      program_name, strerror(errno), file);
-      exit_code = ENOENT; return(ENOENT);
-    }
-
   if ((code = getfiledata(file)) != 0)
     {exit_code = code; return(code);}
 
@@ -1761,6 +1674,7 @@ int main(int argc, char **argv)
     }
 
   /* initialize hash buffer */
+
   if (init_hashin() != 0)
   {
     fprintf(stderr, "\r%s: %s: cannot initialize hash buffer\n",
